% *************************************************************************
% ********************************** OWE **********************************
% An On-line Weighted Ensemble of Regressor Models to Handle Concept Drifts
% *************************************************************************
% Version 1.0 Nov/08/2014
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com or symonesoares@isr.uc.pt
%
% Homepage: http://sites.google.com/site/symonesoares/
%
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An On-line Weighted Ensemble 
%        of Regressor Models to Handle Concept Drifts. Engineering  
%        Applications of Artificial Intelligence, v. 37, p. 392-406, 2015. 
%     url: http://www.sciencedirect.com/science/article/pii/S09521976140024
%
% *************************************************************************
% *************************************************************************
% An Example of the OWE algorithm and the Learn++.NSE* for regression
% using the **FCCU data set
% 
%   *Elwell, R., Polikar, R., 2011. Incremental Learning of Concept Drift in Nonstationary
%    Environments. IEEE Trans. Neural Netw. 22 (October (10)), 1517–1531.
%
%  **Liu, Y., Hu, N., Wang, H., Li, P., 2009. Soft Chemical Analyzer Development Using
%    Adaptive Least-Squares Support Vector Regression With Selective Pruning and
%    Variable Moving Window Size. Ind. Eng. Chem. Res. 48 (May (12)),
%    5731–5741.
%
% *************************************************************************

clear; clc; close all;

% Loading the FCCU data set
% Using only the first output (gasoline concentration)
D      = load('FCCU.txt'); 
input  = D(:,1:6); % attributing the inputs  
output = D(:,7);   % attributing the output  
output = mmnorm(output, 0, 1); % normalizing the output

% Defining the global input parameters
m       = 20;   % window's size 
Theta   = 0.05; % for demarcating correct and incorrect prediction (0 < Theta < 1);
Pruning = 1;    % ensemble pruning is activated
B       = 20;   % maximum number of models in the ensemble

% Calling fOWE fuction...
Kappa   = 0.2;  % discounting factor (0 <= Kappa < 1); Default: 0.2;
Alpha   = 0.04; % factor to include a new model (0 < Alpha < 1);
fprintf('*** Starting fOWE function ***');
[CC_OWE, MSE_OWE, RMSE_OWE, nM_OWE, YR_OWE, YP_OWE, loss_OWE, time_OWE] = fOWE(input, output, m, Kappa, Theta, Alpha, B, Pruning);

% Calling fLearnNSEReg fuction...
a = 0.5; % slope parameter a (Default: 0.5)
b = 10;  % slope parameter b (Default: 10)
fprintf('*** Starting fLearnNSEReg function ***');
[CC, MSE, RMSE, nM, YR, YP, loss, time] = fLearnNSEReg(input, output, m, a, b, Theta, B, Pruning);




